from typing import Dict, Iterable, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class BackBySale(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса "Возврат по идентификатору чека"')
        self.__updateCases()

        servicestub.route('/CSrest/rest/sales/document/<docid>', method='GET', callback=self.sales)

    def sales(self, docid=''):
        self.log.info(f"Запрос документа с идентификатором '{docid}'")
        self.__updateCases()

        methodPath = f'/CSrest/rest/sales/document/{docid}'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        body = json.dumps(self.cases[methodPath])
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'backbysale.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = BackBySale()
