import bottle
import json
import time
import os

from logger import Logger
from singleton import Singleton

class MetroExcise(metaclass=Singleton):

    def __init__(self):
        self.log = Logger()
        self.cases = {}
        self.caseNumber = 0

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса')
        self.__updateCases()
        servicestub.route('/transport/app/entites', method='POST', callback=self.check)

    def check(self):
        self.log.info('Запрос к акцизному сервису')
        self.__updateCases()
        return self.__getResult()

    def __getResult(self):
        if len(self.cases["caseSeq"]) <= self.caseNumber:
            self.caseNumber = 0
        caseId = self.cases["caseSeq"][self.caseNumber]
        self.caseNumber = self.caseNumber + 1

        self.log.info('Используется ответ %s', caseId)
        case = self.cases.get(caseId)
        time.sleep(case.get('timeout', 0))

        response = None
        if 'response' in case:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "xml/" + case['response']), 'rb') as f:
                body = f.read()
            response = bottle.HTTPResponse(status=case.get('status', 200), body=body)
            response.add_header('Content-Type', 'text/xml; charset=utf8')
        else:
            raise bottle.HTTPError(400)

        if response is bottle.HTTPError:
            raise response
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cases.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = MetroExcise()
