from typing import Dict, Iterable, Any
import time, json, os

import bottle
import base64

from logger import Logger
from singleton import Singleton
from servicestub import ServiceStub

class MfgMarking(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса')
        self.__updateCases()

        servicestub.route('/mfgmarking', method='POST', callback=self.handler)

    def handler(self, operation='', location='', uit='' ):
        self.__updateCases()
        body = json.loads(bottle.request.body.read())

        if 'operation' not in body:
            raise bottle.HTTPError(400, 'В запросе нет поля operation')

        if 'uit' not in body:
            raise bottle.HTTPError(400, 'В запросе нет поля uit')

        uit = base64.b64decode(body['uit']).decode('utf-8')
        operation = body['operation'].lower()
        self.log.info("Получение статуса кода маркировки: '%s'" % uit)
        methodPath = '/%s/<location>/%s' %(operation, uit)
        if methodPath not in self.cases:
            methodPath = '/%s/<location>/<uit>' % operation
        self.log.info("Получаем результат для: '%s'" % methodPath)
        return self.__getResult(methodPath)

    def __getResult(self, methodPath):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath][0].get('timeout', 0) / 1000)
        body = json.dumps(self.cases[methodPath][0].get('response'))

        response = bottle.HTTPResponse(status=self.cases[methodPath][0].get('status', 200), body=body)
        response.add_header("Content-Type", "application/json")
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'mfgmarking.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = MfgMarking()
