from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class SoftCheck(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса softcheck')
        self.__updateCases()

        servicestub.route('/dicts/softCheck/<checkId>', method='GET', callback=self.dicts)
        servicestub.route('/sales/softCheck/', method='POST', callback=self.sales)

    def dicts(self, checkId=''):
        self.__updateCases()
        self.log.info("Получение софт-чека с сервера: '%s'" % checkId)
        methodPath = '/dicts/' + checkId
        if methodPath not in self.cases:
            methodPath = '/dicts/<checkId>'

        return self.__getResult(methodPath)

    def sales(self):
        self.__updateCases()
        params = dict(bottle.request.query.decode())
        checkId = params.get('patientIdentifier', '')
        self.log.info("Изменение статуса продажи софт-чека: '%s'" % checkId)
        methodPath = '/sales/' + checkId
        if methodPath not in self.cases:
            methodPath = '/sales/<checkId>'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath].get('timeout', 0))
        body = json.dumps(self.cases[methodPath].get('response'))

        response = bottle.HTTPResponse(status=self.cases[methodPath].get('status', 200), body=body)
        response.add_header("Content-Type", "application/json")
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'softcheck.json'), 'r') as cases:
                self.cases = json.load(cases)
        except Exception as e:
            self.log.error(str(e))


instance = SoftCheck()
